@@ -1,11 +1,10 @@
 /* -*- Mode: C; tab-width: 4 -*- */
 /*-
  * ant --- Chris Langton's generalized turing machine ants (also known
- *         as Greg Turk's turmites) whose tape is the screen
- */
+ *         as Greg Turk's turmites) whose tape is the screen */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)ant.c	5.13 2004/06/23 xlockmore";
+static const char sccsid[] = "@(#)ant.c	5.24 2007/01/18 xlockmore";
 
 #endif
 
@@ -25,6 +24,7 @@ static const char sccsid[] = "@(#)ant.c	5.13 2004/06/23 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
+ * 18-Jan-2007: Added vertical option.
  * 01-Nov-2000: Allocation checks
  * 10-May-1997: Compatible with xscreensaver
  * 16-Apr-1997: -neighbors 3 and 8 added
@@ -76,6 +76,10 @@ static const char sccsid[] = "@(#)ant.c	5.13 2004/06/23 xlockmore";
 
 #ifdef MODE_ant
 
+#define FONT_HEIGHT 19
+#define FONT_WIDTH 15
+#define DEF_LABEL "True"
+
 /*-
  * neighbors of 0 randomizes it for 3, 4, 6, 8, 12 (last 2 are less likely)
  */
@@ -85,15 +89,20 @@ static const char sccsid[] = "@(#)ant.c	5.13 2004/06/23 xlockmore";
 #define DEF_TRUCHET  "False"
 #define DEF_EYES  "False"
 #define DEF_SHARPTURN  "False"
+#define DEF_VERTICAL "False"
 
+static Bool label;
 static int  neighbors;
 static char *rule;
 static Bool truchet;
 static Bool eyes;
 static Bool sharpturn;
+static Bool vertical;
 
 static XrmOptionDescRec opts[] =
 {
+	{(char *) "-label", (char *) ".ant.label", XrmoptionNoArg, (caddr_t) "on"},
+	{(char *) "+label", (char *) ".ant.label", XrmoptionNoArg, (caddr_t) "off"},
 	{(char *) "-neighbors", (char *) ".ant.neighbors", XrmoptionSepArg, (caddr_t) NULL},
 	{(char *) "-rule", (char *) ".ant.rule", XrmoptionSepArg, (caddr_t) NULL},
 	{(char *) "-truchet", (char *) ".ant.truchet", XrmoptionNoArg, (caddr_t) "on"},
@@ -101,23 +110,29 @@ static XrmOptionDescRec opts[] =
 	{(char *) "-eyes", (char *) ".ant.eyes", XrmoptionNoArg, (caddr_t) "on"},
 	{(char *) "+eyes", (char *) ".ant.eyes", XrmoptionNoArg, (caddr_t) "off"},
 	{(char *) "-sharpturn", (char *) ".ant.sharpturn", XrmoptionNoArg, (caddr_t) "on"},
-	{(char *) "+sharpturn", (char *) ".ant.sharpturn", XrmoptionNoArg, (caddr_t) "off"}
+	{(char *) "+sharpturn", (char *) ".ant.sharpturn", XrmoptionNoArg, (caddr_t) "off"},
+	{(char *) "-vertical", (char *) ".ant.vertical", XrmoptionNoArg, (caddr_t) "on"},
+	{(char *) "+vertical", (char *) ".ant.vertical", XrmoptionNoArg, (caddr_t) "off"}
 };
 static argtype vars[] =
 {
+	{(void *) & label, (char *) "label", (char *) "Label", (char *) DEF_LABEL, t_Bool},
 	{(void *) & neighbors, (char *) "neighbors", (char *) "Neighbors", (char *) DEF_NEIGHBORS, t_Int},
 	{(void *) & rule, (char *) "rule", (char *) "Rule", (char *) DEF_RULE, t_String},
 	{(void *) & truchet, (char *) "truchet", (char *) "Truchet", (char *) DEF_TRUCHET, t_Bool},
 	{(void *) & eyes, (char *) "eyes", (char *) "Eyes", (char *) DEF_EYES, t_Bool},
-	{(void *) & sharpturn, (char *) "sharpturn", (char *) "SharpTurn", (char *) DEF_SHARPTURN, t_Bool}
+	{(void *) & sharpturn, (char *) "sharpturn", (char *) "SharpTurn", (char *) DEF_SHARPTURN, t_Bool},
+	{(void *) & vertical, (char *) "vertical", (char *) "Vertical", (char *) DEF_VERTICAL, t_Bool}
 };
 static OptionStruct desc[] =
 {
+	{(char *) "-/+label", (char *) "turn on/off rule labeling"},
 	{(char *) "-neighbors num", (char *) "squares 4 or 8, hexagons 6, triangles 3 or 12"},
 	{(char *) "-rule string", (char *) "binary string for Turk's Ant"},
 	{(char *) "-/+truchet", (char *) "turn on/off Truchet lines"},
 	{(char *) "-/+eyes", (char *) "turn on/off eyes"},
-	{(char *) "-/+sharpturn", (char *) "turn on/off sharp turns (6, 8 or 12 neighbors only)"}
+	{(char *) "-/+sharpturn", (char *) "turn on/off sharp turns (6, 8 or 12 neighbors only)"},
+	{(char *) "-/+vertical", (char *) "change orientation for hexagons and triangles"}
 };
 
 ModeSpecOpt ant_opts =
@@ -160,7 +175,7 @@ typedef struct {
 } antstruct;
 
 typedef struct {
-	Bool        painted;
+	Bool        painted, vertical;
 	int         neighbors;
 	int         generation;
 	int         xs, ys;
@@ -182,6 +197,8 @@ typedef struct {
 	unsigned char colors[NUMSTIPPLES - 1];
 	GC          stippledGC;
 	Pixmap      pixmaps[NUMSTIPPLES - 1];
+	int         labelOffsetX, labelOffsetY;
+	char        ruleString[40];
 	union {
 		XPoint      hexagon[7];		/* Need more than 6 for truchet */
 		XPoint      triangle[2][4];	/* Need more than 3 for truchet */
@@ -477,8 +494,13 @@ fillcell(ModeInfo * mi, GC gc, int col, int row)
 	if (ap->neighbors == 6) {
 		int         ccol = 2 * col + !(row & 1), crow = 2 * row;
 
-		ap->shape.hexagon[0].x = ap->xb + ccol * ap->xs;
-		ap->shape.hexagon[0].y = ap->yb + crow * ap->ys;
+		if (ap->vertical) {
+			ap->shape.hexagon[0].x = ap->xb + ccol * ap->xs;
+			ap->shape.hexagon[0].y = ap->yb + crow * ap->ys;
+		} else {
+			ap->shape.hexagon[0].y = ap->xb + ccol * ap->xs;
+			ap->shape.hexagon[0].x = ap->yb + crow * ap->ys;
+		}
 		if (ap->xs == 1 && ap->ys == 1)
 			XDrawPoint(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
 			ap->shape.hexagon[0].x, ap->shape.hexagon[0].y);
@@ -490,21 +512,36 @@ fillcell(ModeInfo * mi, GC gc, int col, int row)
 		ap->xb + ap->xs * col, ap->yb + ap->ys * row,
 	 	ap->xs - (ap->xs > 3), ap->ys - (ap->ys > 3));
 	} else {		/* TRI */
-		int         orient = (col + row) % 2;	/* O left 1 right */
+		int orient = (col + row) % 2;	/* O left 1 right */
+		Bool small = (ap->xs <= 3 || ap->ys <= 3);
 
-		ap->shape.triangle[orient][0].x = ap->xb + col * ap->xs;
-		ap->shape.triangle[orient][0].y = ap->yb + row * ap->ys;
-		if (ap->xs <= 3 || ap->ys <= 3)
-			XDrawPoint(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
-			((orient) ? -1 : 1) + ap->shape.triangle[orient][0].x,
-				       ap->shape.triangle[orient][0].y);
-		else {
-			if (orient)
-				ap->shape.triangle[orient][0].x += (ap->xs / 2 - 1);
+		if (ap->vertical) {
+			ap->shape.triangle[orient][0].x = ap->xb + col * ap->xs;
+			ap->shape.triangle[orient][0].y = ap->yb + row * ap->ys;
+			if (small)
+				ap->shape.triangle[orient][0].x +=
+					((orient) ? -1 : 1);
 			else
-				ap->shape.triangle[orient][0].x -= (ap->xs / 2 - 1);
+				ap->shape.triangle[orient][0].x +=
+					(ap->xs / 2  - 1) * ((orient) ? 1 : -1);
+		} else {
+			ap->shape.triangle[orient][0].y = ap->xb + col * ap->xs;
+			ap->shape.triangle[orient][0].x = ap->yb + row * ap->ys;
+			if (small)
+				ap->shape.triangle[orient][0].y +=
+					((orient) ? -1 : 1);
+			else
+				ap->shape.triangle[orient][0].y +=
+					(ap->xs / 2  - 1) * ((orient) ? 1 : -1);
+		}
+		if (small)
+			XDrawPoint(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
+				ap->shape.triangle[orient][0].x,
+				ap->shape.triangle[orient][0].y);
+		else {
 			XFillPolygon(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
-				     ap->shape.triangle[orient], 3, Convex, CoordModePrevious);
+				ap->shape.triangle[orient], 3,
+				Convex, CoordModePrevious);
 		}
 	}
 }
@@ -526,13 +563,26 @@ truchetcell(ModeInfo * mi, int col, int row, int truchetstate)
 			hex.y = ap->yb + crow * ap->ys - (int) ((double) ap->ys / 2.0) - 1;
 			for (side = 0; side < 6; side++) {
 				if (side) {
-					hex.x += ap->shape.hexagon[side].x;
-					hex.y += ap->shape.hexagon[side].y;
+					if (ap->vertical) {
+						hex.x += ap->shape.hexagon[side].x;
+						hex.y += ap->shape.hexagon[side].y;
+					} else {
+						hex.y += ap->shape.hexagon[side].x;
+						hex.x += ap->shape.hexagon[side].y;
+					}
 				}
 				if (truchetstate == side % 2)
-					XDrawArc(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
-						 hex.x, hex.y, ap->xs, ap->ys,
-						 ((570 - (side * 60) + fudge) % 360) * 64, (120 - 2 * fudge) * 64);
+					if (ap->vertical) {
+					    XDrawArc(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
+						hex.x, hex.y, ap->xs, ap->ys,
+						((570 - (side * 60) + fudge) % 360) * 64,
+						(120 - 2 * fudge) * 64);
+					} else {
+					    XDrawArc(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
+						hex.y, hex.x, ap->ys, ap->xs,
+						((300 + (side * 60) + fudge) % 360) * 64,
+						(120 - 2 * fudge) * 64);
+					}
 			}
 		} else {
 			/* Very crude approx of Sqrt 3, so it will not cause drawing errors. */
@@ -540,11 +590,21 @@ truchetcell(ModeInfo * mi, int col, int row, int truchetstate)
 			hex.y = ap->yb + crow * ap->ys - (int) ((double) ap->ys * 1.6 / 2.0) - 1;
 			for (side = 0; side < 6; side++) {
 				if (side) {
-					hex.x += ap->shape.hexagon[side].x;
-					hex.y += ap->shape.hexagon[side].y;
+					if (ap->vertical) {
+						hex.x += ap->shape.hexagon[side].x;
+						hex.y += ap->shape.hexagon[side].y;
+					} else {
+						hex.y += ap->shape.hexagon[side].x;
+						hex.x += ap->shape.hexagon[side].y;
+					}
+				}
+				if (ap->vertical) {
+					hex2.x = hex.x + ap->shape.hexagon[side + 1].x / 2;
+					hex2.y = hex.y + ap->shape.hexagon[side + 1].y / 2 + 1;
+				} else {
+					hex2.y = hex.y + ap->shape.hexagon[side + 1].x / 2;
+					hex2.x = hex.x + ap->shape.hexagon[side + 1].y / 2 + 1;
 				}
-				hex2.x = hex.x + ap->shape.hexagon[side + 1].x / 2;
-				hex2.y = hex.y + ap->shape.hexagon[side + 1].y / 2 + 1;
 				/* Lots of fudging here */
 				if (side == 0) {
 					hex2.x += 1;
@@ -565,10 +625,17 @@ truchetcell(ModeInfo * mi, int col, int row, int truchetstate)
 				}
 				if (truchetstate == side % 3)
 					/* Crude approx of 120 deg, so it will not cause drawing errors. */
-					XDrawArc(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
+					if (ap->vertical) {
+					    XDrawArc(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
 						 hex2.x, hex2.y,
 						 (int) ((double) ap->xs * 1.5), (int) ((double) ap->ys * 1.5),
 						 ((555 - (side * 60)) % 360) * 64, 90 * 64);
+					} else {
+					    XDrawArc(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
+						 hex2.y, hex2.x,
+						 (int) ((double) ap->ys * 1.5), (int) ((double) ap->xs * 1.5),
+						 ((345 + (side * 60)) % 360) * 64, 90 * 64);
+					}
 			}
 		}
 	} else if (ap->neighbors == 4) {
@@ -611,20 +678,41 @@ truchetcell(ModeInfo * mi, int col, int row, int truchetstate)
 		}
 		for (side = 0; side < 3; side++) {
 			if (side > 0) {
-				tri.x += ap->shape.triangle[orient][side].x;
-				tri.y += ap->shape.triangle[orient][side].y;
+				if (ap->vertical) {
+					tri.x += ap->shape.triangle[orient][side].x;
+					tri.y += ap->shape.triangle[orient][side].y;
+				} else {
+					tri.y += ap->shape.triangle[orient][side].x;
+					tri.x += ap->shape.triangle[orient][side].y;
+				}
 			}
 			if (truchetstate == side) {
-				if (orient)
-					ang = (510 - side * 120) % 360;		/* Right */
-				else
-					ang = (690 - side * 120) % 360;		/* Left */
-				XDrawArc(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
-				  (int) (tri.x - ap->xs * fudge2 / 2),
-          (int) (tri.y - 3 * ap->ys * fudge2 / 4),
-				  (unsigned int) (ap->xs * fudge2),
-          (unsigned int) (3 * ap->ys * fudge2 / 2),
-				  (ang + fudge) * 64, (60 - 2 * fudge) * 64);
+				if (ap->vertical) {
+					if (orient)
+						ang = (510 - side * 120) % 360;		/* Right */
+					else
+						ang = (690 - side * 120) % 360;		/* Left */
+				} else {
+					if (orient)
+						ang = (60 + side * 120) % 360;		/* Right */
+					else
+						ang = (240 + side * 120) % 360;		/* Left */
+				}
+				if (ap->vertical) {
+					XDrawArc(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
+				    (int) (tri.x - ap->xs * fudge2 / 2),
+				    (int) (tri.y - 3 * ap->ys * fudge2 / 4),
+				    (unsigned int) (ap->xs * fudge2),
+				    (unsigned int) (3 * ap->ys * fudge2 / 2),
+				    (ang + fudge) * 64, (60 - 2 * fudge) * 64);
+				} else {
+					XDrawArc(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
+				    (int) (tri.y - 3 * ap->ys * fudge2 / 4),
+				    (int) (tri.x - ap->xs * fudge2 / 2),
+				    (unsigned int) (3 * ap->ys * fudge2 / 2),
+				    (unsigned int) (ap->xs * fudge2),
+				    (ang + fudge) * 64, (60 - 2 * fudge) * 64);
+				}
 			}
 		}
 	}
@@ -890,6 +978,8 @@ getTable(ModeInfo * mi, int i)
 		(void) fprintf(stdout,
 			       "ants %d, neighbors %d, table number %d, colors %d, states %d\n",
 			  ap->n, ap->neighbors, i, ap->ncolors, ap->nstates);
+	if (label)
+		(void) sprintf(ap->ruleString, "table number %d", i);
 	for (j = 0; j < total; j++) {
 		status = &(ap->machine[j]);
 		status->color = *patptr++;
@@ -963,6 +1053,8 @@ getTurk(ModeInfo * mi, int logClass, int number)
 		(void) fprintf(stdout,
 			"ants %d, neighbors %d, Turk's number 0x%x, colors %d\n",
 			ap->n, ap->neighbors, number, ap->ncolors);
+	if (label)
+		(void) sprintf(ap->ruleString, "Turk's number 0x%x", number);
 }
 
 static void
@@ -1090,6 +1182,17 @@ init_ant(ModeInfo * mi)
 	} else if (ap->n < MINANTS)
 		ap->n = MINANTS;
 
+	if (MI_IS_FULLRANDOM(mi)) {
+		ap->truchet = (Bool) (LRAND() & 1);
+		ap->eyes = (Bool) (LRAND() & 1);
+		ap->sharpturn = (Bool) (LRAND() & 1);
+		ap->vertical = (Bool) (LRAND() & 1);
+	} else {
+		ap->truchet = truchet;
+		ap->eyes = eyes;
+		ap->sharpturn = sharpturn;
+		ap->vertical = vertical;
+	}
 	ap->width = MI_WIDTH(mi);
 	ap->height = MI_HEIGHT(mi);
 
@@ -1111,6 +1214,10 @@ init_ant(ModeInfo * mi)
 	if (ap->neighbors == 6) {
 		int         nccols, ncrows;
 
+		if (!ap->vertical) {
+			ap->height = MI_WIDTH(mi);
+			ap->width = MI_HEIGHT(mi);
+		}
 		if (ap->width < 8)
 			ap->width = 8;
 		if (ap->height < 8)
@@ -1137,8 +1244,19 @@ init_ant(ModeInfo * mi)
 		ap->xb = (ap->width - ap->xs * nccols) / 2 + ap->xs / 2;
 		ap->yb = (ap->height - ap->ys * (ncrows / 2) * 2) / 2 + ap->ys - 2;
 		for (i = 0; i < 6; i++) {
-			ap->shape.hexagon[i].x = (ap->xs - 1) * hexagonUnit[i].x;
-			ap->shape.hexagon[i].y = ((ap->ys - 1) * hexagonUnit[i].y / 2) * 4 / 3;
+			if (ap->vertical) {
+				ap->shape.hexagon[i].x =
+					(ap->xs - 1) * hexagonUnit[i].x;
+				ap->shape.hexagon[i].y =
+					((ap->ys - 1) * hexagonUnit[i].y /
+					2) * 4 / 3;
+			} else {
+				ap->shape.hexagon[i].y =
+					(ap->xs - 1) * hexagonUnit[i].x;
+				ap->shape.hexagon[i].x =
+					((ap->ys - 1) * hexagonUnit[i].y /
+					2) * 4 / 3;
+			}
 		}
 		/* Avoid array bounds read of hexagonUnit */
 		ap->shape.hexagon[6].x = 0;
@@ -1164,8 +1282,12 @@ init_ant(ModeInfo * mi)
 		ap->xb = (ap->width - ap->xs * ap->ncols) / 2;
 		ap->yb = (ap->height - ap->ys * ap->nrows) / 2;
 	} else {		/* TRI */
-		int         orient;
+		int orient;
 
+		if (!ap->vertical) {
+			ap->height = MI_WIDTH(mi);
+			ap->width = MI_HEIGHT(mi);
+		}
 		if (ap->width < 2)
 			ap->width = 2;
 		if (ap->height < 2)
@@ -1191,10 +1313,17 @@ init_ant(ModeInfo * mi)
 		ap->yb = (ap->height - ap->ys * ap->nrows) / 2 + ap->ys;
 		for (orient = 0; orient < 2; orient++) {
 			for (i = 0; i < 3; i++) {
-				ap->shape.triangle[orient][i].x =
-					(ap->xs - 2) * triangleUnit[orient][i].x;
-				ap->shape.triangle[orient][i].y =
-					(ap->ys - 2) * triangleUnit[orient][i].y;
+				if (ap->vertical) {
+					ap->shape.triangle[orient][i].x =
+						(ap->xs - 2) * triangleUnit[orient][i].x;
+					ap->shape.triangle[orient][i].y =
+						(ap->ys - 2) * triangleUnit[orient][i].y;
+				} else {
+					ap->shape.triangle[orient][i].y =
+						(ap->xs - 2) * triangleUnit[orient][i].x;
+					ap->shape.triangle[orient][i].x =
+						(ap->ys - 2) * triangleUnit[orient][i].y;
+				}
 			}
 			/* Avoid array bounds read of triangleUnit */
 			ap->shape.triangle[orient][3].x = 0;
@@ -1206,15 +1335,8 @@ init_ant(ModeInfo * mi)
 	MI_CLEARWINDOW(mi);
 	ap->painted = False;
 
-	if (MI_IS_FULLRANDOM(mi)) {
-		ap->truchet = (Bool) (LRAND() & 1);
-		ap->eyes = (Bool) (LRAND() & 1);
-		ap->sharpturn = (Bool) (LRAND() & 1);
-	} else {
-		ap->truchet = truchet;
-		ap->eyes = eyes;
-		ap->sharpturn = sharpturn;
-	}
+	ap->labelOffsetX = NRAND(8);
+	ap->labelOffsetY = NRAND(8);
 	parseRule(mi);
 	if (MI_NPIXELS(mi) > 2)
 		for (i = 0; i < (int) ap->ncolors - 1; i++)
@@ -1357,6 +1479,23 @@ draw_ant(ModeInfo * mi)
 			}
 		}
 	}
+	if (label) {
+		int size = MAX(MIN(MI_WIDTH(mi), MI_HEIGHT(mi)) - 1, 1);
+
+		if (size >= 10 * FONT_WIDTH) {
+			/* hard code these to corners */
+			XSetForeground(MI_DISPLAY(mi), MI_GC(mi),
+				MI_WHITE_PIXEL(mi));
+			/*XDrawString(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
+				16 + ap->labelOffsetX,
+				16 + ap->labelOffsetY + FONT_HEIGHT,
+				ap->ruleString, strlen(ap->ruleString)); */
+			XDrawString(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
+				16 + ap->labelOffsetX, MI_HEIGHT(mi) - 16 -
+				ap->labelOffsetY - FONT_HEIGHT / 2,
+				ap->ruleString, strlen(ap->ruleString));
+		}
+	}
 }
 
 void
