@@ -2,7 +2,7 @@
 /* bug --- Michael Palmiter's simulated evolution */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)bug.c	5.00 2000/11/01 xlockmore";
+static const char sccsid[] = "@(#)bug.c	5.24 2007/01/18 xlockmore";
 
 #endif
 
@@ -22,6 +22,7 @@ static const char sccsid[] = "@(#)bug.c	5.00 2000/11/01 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
+ * 18-Jan-2007: Added vertical option.
  * 01-Nov-2000: Allocation checks
  * 10-May-1997: Compatible with xscreensaver
  * 24-Aug-1995: Coded from A.K. Dewdney's, "Computer Recreations", Scientific
@@ -57,25 +58,31 @@ static const char sccsid[] = "@(#)bug.c	5.00 2000/11/01 xlockmore";
 
 #define DEF_NEIGHBORS  "0"      /* choose random value */
 #define DEF_EYES  "False"
+#define DEF_VERTICAL "False"
 
 static int  neighbors;
 static Bool eyes;
+static Bool vertical;
 
 static XrmOptionDescRec opts[] =
 {
 	{(char *) "-neighbors", (char *) ".bug.neighbors", XrmoptionSepArg, (caddr_t) NULL},
 	{(char *) "-eyes", (char *) ".bug.eyes", XrmoptionNoArg, (caddr_t) "on"},
 	{(char *) "+eyes", (char *) ".bug.eyes", XrmoptionNoArg, (caddr_t) "off"},
+	{(char *) "-vertical", (char *) ".bug.vertical", XrmoptionNoArg, (caddr_t) "on"},
+	{(char *) "+vertical", (char *) ".bug.vertical", XrmoptionNoArg, (caddr_t) "off"}
 };
 static argtype vars[] =
 {
-	{(caddr_t *) & neighbors, (char *) "neighbors", (char *) "Neighbors", (char *) DEF_NEIGHBORS, t_Int},
-	{(caddr_t *) & eyes, (char *) "eyes", (char *) "Eyes", (char *) DEF_EYES, t_Bool},
+	{(void *) & neighbors, (char *) "neighbors", (char *) "Neighbors", (char *) DEF_NEIGHBORS, t_Int},
+	{(void *) & eyes, (char *) "eyes", (char *) "Eyes", (char *) DEF_EYES, t_Bool},
+	{(void *) & vertical, (char *) "vertical", (char *) "Vertical", (char *) DEF_VERTICAL, t_Bool}
 };
 static OptionStruct desc[] =
 {
-	{(char *) "-neighbors num", (char *) "squares 4 or 8, hexagons 6"},
-	{(char *) "-/+eyes", (char *) "turn on/off eyes"}
+	{(char *) "-neighbors num", (char *) "squares 4 or 8, hexagons 6, triangles 3 or 12"},
+	{(char *) "-/+eyes", (char *) "turn on/off eyes"},
+	{(char *) "-/+vertical", (char *) "change orientation for hexagons and triangles"}
 };
 
 ModeSpecOpt bug_opts =
@@ -141,7 +148,7 @@ typedef struct _BugList {
 } BugList;
 
 typedef struct {
-	Bool        painted;
+	Bool        painted, vertical;
 	int         neighbors;
 	int         nbugs;	/* Number of bugs */
 	int         eden;	/* Does the garden exist? */
@@ -186,38 +193,7 @@ position_of_neighbor(bugfarmstruct * bp, int dir, int *pcol, int *prow)
 
 	/* NO WRAPING */
 
-	if (bp->neighbors == 6) {
-		switch (dir) {
-			case 0:
-				col = col + 1;
-				break;
-			case 60:
-				if (!(row & 1))
-					col = col + 1;
-				row = row - 1;
-				break;
-			case 120:
-				if (row & 1)
-					col = col - 1;
-				row = row - 1;
-				break;
-			case 180:
-				col = col - 1;
-				break;
-			case 240:
-				if (row & 1)
-					col = col - 1;
-				row = row + 1;
-				break;
-			case 300:
-				if (!(row & 1))
-					col = col + 1;
-				row = row + 1;
-				break;
-			default:
-				(void) fprintf(stderr, "wrong direction %d\n", dir);
-		}
-	} else if (bp->neighbors == 4 || bp->neighbors == 8) {
+	if (bp->neighbors == 4 || bp->neighbors == 6 || bp->neighbors == 8) {
 		switch (dir) {
 			case 0:
 				col = col + 1;
@@ -226,9 +202,19 @@ position_of_neighbor(bugfarmstruct * bp, int dir, int *pcol, int *prow)
 				col = col + 1;
 				row = row - 1;
 				break;
+			case 60:
+				if (!(row & 1))
+					col = col + 1;
+				row = row - 1;
+				break;
 			case 90:
 				row = row - 1;
 				break;
+			case 120:
+				if (row & 1)
+					col = col - 1;
+				row = row - 1;
+				break;
 			case 135:
 				col = col - 1;
 				row = row - 1;
@@ -240,9 +226,19 @@ position_of_neighbor(bugfarmstruct * bp, int dir, int *pcol, int *prow)
 				col = col - 1;
 				row = row + 1;
 				break;
+			case 240:
+				if (row & 1)
+					col = col - 1;
+				row = row + 1;
+				break;
 			case 270:
 				row = row + 1;
 				break;
+			case 300:
+				if (!(row & 1))
+					col = col + 1;
+				row = row + 1;
+				break;
 			case 315:
 				col = col + 1;
 				row = row + 1;
@@ -607,9 +603,17 @@ drawabacterium(ModeInfo * mi, int col, int row, unsigned char color)
 	else
 		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_BLACK_PIXEL(mi));
 	grid2screen(bp, col, row, &ccol, &crow);
-	XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
-	       bp->xb + bp->xs * ccol, bp->yb + bp->ys * crow,
-	       bp->xs, bp->ys);
+	if (bp->vertical) {
+		XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
+			bp->xb + bp->xs * ccol,
+			bp->yb + bp->ys * crow,
+			bp->xs, bp->ys);
+	} else {
+		XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
+			bp->yb + bp->ys * crow,
+			bp->xb + bp->xs * ccol,
+			bp->ys, bp->xs);
+	}
 }
 
 static void
@@ -638,98 +642,175 @@ drawabug(ModeInfo * mi, int col, int row, unsigned int color, int direction)
 	grid2screen(bp, col, row, &ccol, &crow);
 	ccol = bp->xb + bp->xs * (ccol - 1);
 	crow = bp->yb + bp->ys * (crow - 1);
-	XFillRectangle(display, window, gc,
-		ccol, crow, 3 * bp->xs, 3 * bp->ys);
+	if (bp->vertical) {
+		XFillRectangle(display, window, gc,
+			ccol, crow, 3 * bp->xs, 3 * bp->ys);
+	} else {
+		XFillRectangle(display, window, gc,
+			crow, ccol, 3 * bp->ys, 3 * bp->xs);
+	}
 
 	if (bp->eyes && bp->xs >= 1 && bp->ys >= 1) { /* Draw Eyes */
 		XSetForeground(display, MI_GC(mi), MI_BLACK_PIXEL(mi));
 		switch (direction) {
-			case 0:
+		case 0:
+			if (bp->vertical) {
 				XDrawPoint(display, window, MI_GC(mi),
 					ccol + 3 * bp->xs - 2,
 					crow + 3 * bp->ys / 2 - 1);
 				XDrawPoint(display, window, MI_GC(mi),
 					ccol + 3 * bp->xs - 2,
 					crow + 3 * bp->ys / 2 + 1);
-					break;
-			case 20:
-			case 30:
-			case 40:
-			case 45:
-			case 60:
+			} else {
+				XDrawPoint(display, window, MI_GC(mi),
+					crow + 3 * bp->ys / 2 - 1,
+					ccol + 3 * bp->xs - 2);
+				XDrawPoint(display, window, MI_GC(mi),
+					crow + 3 * bp->ys / 2 + 1,
+					ccol + 3 * bp->xs - 2);
+			}
+			break;
+		case 20:
+		case 30:
+		case 40:
+		case 45:
+		case 60:
+			if (bp->vertical) {
 				XDrawPoint(display, window, MI_GC(mi),
 					ccol + 3 * bp->xs - 3,
 					crow + 1);
 				XDrawPoint(display, window, MI_GC(mi),
 					ccol + 3 * bp->xs - 2,
 					crow + 2);
-				break;
-			case 80:
-			case 90:
-			case 100:
+			} else {
+				XDrawPoint(display, window, MI_GC(mi),
+					crow + 1,
+					ccol + 3 * bp->xs - 3);
+				XDrawPoint(display, window, MI_GC(mi),
+					crow + 2,
+					ccol + 3 * bp->xs - 2);
+			}
+			break;
+		case 80:
+		case 90:
+		case 100:
+			if (bp->vertical) {
 				XDrawPoint(display, window, MI_GC(mi),
 					ccol + 3 * bp->xs / 2 - 1,
 					crow + 1);
 				XDrawPoint(display, window, MI_GC(mi),
 					ccol + 3 * bp->xs / 2 + 1,
 					crow + 1);
-				break;
-			case 120:
-			case 135:
-			case 140:
-			case 150:
-			case 160:
+			} else {
+				XDrawPoint(display, window, MI_GC(mi),
+					crow + 1,
+					ccol + 3 * bp->xs / 2 - 1);
+				XDrawPoint(display, window, MI_GC(mi),
+					crow + 1,
+					ccol + 3 * bp->xs / 2 + 1);
+			}
+			break;
+		case 120:
+		case 135:
+		case 140:
+		case 150:
+		case 160:
+			if (bp->vertical) {
 				XDrawPoint(display, window, MI_GC(mi),
 					ccol + 2,
 					crow + 1);
 				XDrawPoint(display, window, MI_GC(mi),
 					ccol + 1,
-						crow + 2);
-				break;
-			case 180:
+					crow + 2);
+			} else {
+				XDrawPoint(display, window, MI_GC(mi),
+					crow + 1,
+					ccol + 2);
+				XDrawPoint(display, window, MI_GC(mi),
+					crow + 2,
+					ccol + 1);
+			}
+			break;
+		case 180:
+			if (bp->vertical) {
 				XDrawPoint(display, window, MI_GC(mi),
 					ccol + 1,
 					crow + 3 * bp->ys / 2 - 1);
 				XDrawPoint(display, window, MI_GC(mi),
 					ccol + 1,
 					crow + 3 * bp->ys / 2 + 1);
-				break;
-			case 200:
-			case 210:
-			case 220:
-			case 225:
-			case 240:
+			} else {
+				XDrawPoint(display, window, MI_GC(mi),
+					crow + 3 * bp->ys / 2 - 1,
+					ccol + 1);
+				XDrawPoint(display, window, MI_GC(mi),
+					crow + 3 * bp->ys / 2 + 1,
+					ccol + 1);
+			}
+			break;
+		case 200:
+		case 210:
+		case 220:
+		case 225:
+		case 240:
+			if (bp->vertical) {
 				XDrawPoint(display, window, MI_GC(mi),
 					ccol + 2,
 					crow + 3 * bp->ys - 2);
 				XDrawPoint(display, window, MI_GC(mi),
 					ccol + 1,
 					crow + 3 * bp->ys - 3);
-				break;
-			case 260:
-			case 270:
-			case 280:
+			} else {
+				XDrawPoint(display, window, MI_GC(mi),
+					crow + 3 * bp->ys - 2,
+					ccol + 2);
+				XDrawPoint(display, window, MI_GC(mi),
+					crow + 3 * bp->ys - 3,
+					ccol + 1);
+			}
+			break;
+		case 260:
+		case 270:
+		case 280:
+			if (bp->vertical) {
 				XDrawPoint(display, window, MI_GC(mi),
 					ccol + 3 * bp->xs / 2 - 1,
 					crow + 3 * bp->ys - 2);
 				XDrawPoint(display, window, MI_GC(mi),
 					ccol + 3 * bp->xs / 2 + 1,
 					crow + 3 * bp->ys - 2);
-				break;
-			case 300:
-			case 315:
-			case 320:
-			case 330:
-			case 340:
+			} else {
+				XDrawPoint(display, window, MI_GC(mi),
+					crow + 3 * bp->ys - 2,
+					ccol + 3 * bp->xs / 2 - 1);
+				XDrawPoint(display, window, MI_GC(mi),
+					crow + 3 * bp->ys - 2,
+					ccol + 3 * bp->xs / 2 + 1);
+			}
+			break;
+		case 300:
+		case 315:
+		case 320:
+		case 330:
+		case 340:
+			if (bp->vertical) {
 				XDrawPoint(display, window, MI_GC(mi),
 					ccol + 3 * bp->xs - 3,
 					crow + 3 * bp->ys - 2);
 				XDrawPoint(display, window, MI_GC(mi),
 					ccol + 3 * bp->xs - 2,
 					crow + 3 * bp->ys - 3);
-				break;
-			default:
-				(void) fprintf(stderr, "wrong direction %d\n", direction);
+			} else {
+				XDrawPoint(display, window, MI_GC(mi),
+					crow + 3 * bp->ys - 2,
+					ccol + 3 * bp->xs - 3);
+				XDrawPoint(display, window, MI_GC(mi),
+					crow + 3 * bp->ys - 3,
+					ccol + 3 * bp->xs - 2);
+			}
+			break;
+		default:
+			(void) fprintf(stderr, "wrong direction %d\n", direction);
 		}
 	}
 }
@@ -742,9 +823,17 @@ eraseabug(ModeInfo * mi, int col, int row)
 
 	XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_BLACK_PIXEL(mi));
 	grid2screen(bp, col, row, &ccol, &crow);
-	XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
-		bp->xb + bp->xs * (ccol - 1), bp->yb + bp->ys * (crow - 1),
-		3 * bp->xs, 3 * bp->ys);
+	if (bp->vertical) {
+		XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
+			bp->xb + bp->xs * (ccol - 1),
+			bp->yb + bp->ys * (crow - 1),
+			3 * bp->xs, 3 * bp->ys);
+	} else {
+		XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
+			bp->yb + bp->ys * (crow - 1),
+			bp->xb + bp->xs * (ccol - 1),
+			3 * bp->ys, 3 * bp->xs);
+	}
 }
 
 static void
@@ -1063,6 +1152,15 @@ init_bug(ModeInfo * mi)
 	}
 	bp = &bugfarms[MI_SCREEN(mi)];
 
+	if (MI_IS_FULLRANDOM(mi)) {
+		bp->eyes = (Bool) (LRAND() & 1);
+		bp->vertical = (Bool) (LRAND() & 1);
+	} else {
+		bp->eyes = eyes;
+		bp->vertical = vertical;
+	}
+	bp->eyes = (bp->eyes && bp->xs > 1 && bp->ys > 1);
+
 	if (MI_NPIXELS(mi) <= 3) {
 		if (bp->stippledGC == None) {
 			gcv.fill_style = FillOpaqueStippled;
@@ -1107,6 +1205,13 @@ init_bug(ModeInfo * mi)
 
 	bp->width = MI_WIDTH(mi);
 	bp->height = MI_HEIGHT(mi);
+
+	if (!bp->vertical && bp->neighbors != 4 && bp->neighbors != 8) {
+		bp->height = MI_WIDTH(mi);
+		bp->width = MI_HEIGHT(mi);
+	} else {
+		bp->vertical = True;
+	}
 	if (bp->width < 8)
 		bp->width = 8;
 	if (bp->height < 8)
@@ -1191,13 +1296,6 @@ init_bug(ModeInfo * mi)
 	MI_CLEARWINDOW(mi);
 	bp->painted = False;
 
-	if (MI_IS_FULLRANDOM(mi)) {
-		bp->eyes = (Bool) (LRAND() & 1);
-	} else {
-		bp->eyes = eyes;
-	}
-	bp->eyes = (bp->eyes && bp->xs > 1 && bp->ys > 1);
-
 	/* Try to make bugs, but if can not, exit */
 	while (bp->nbugs < nbugs && ntries < 2 * nbugs) {
 		row = NRAND(bp->nrows);
@@ -1380,7 +1478,8 @@ draw_bug(ModeInfo * mi)
 	makebacteria(mi, FOODPERCYCLE, 0, 0, bp->ncols, bp->nrows, True);
 	if (bp->eden)
 		makebacteria(mi, FOODPERCYCLE,
-			     bp->edenstartx, bp->edenstarty, bp->edenwidth, bp->edenheight, True);
+			bp->edenstartx, bp->edenstarty,
+			bp->edenwidth, bp->edenheight, True);
 	if (!bp->nbugs || bp->generation >= MI_CYCLES(mi))
 		init_bug(mi);
 	bp->generation++;
