@@ -2,7 +2,7 @@
 /* demon --- David Griffeath's cellular automata */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)demon.c	5.00 2000/11/01 xlockmore";
+static const char sccsid[] = "@(#)demon.c	5.24 2007/01/18 xlockmore";
 
 #endif
 
@@ -22,6 +22,7 @@ static const char sccsid[] = "@(#)demon.c	5.00 2000/11/01 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
+ * 18-Jan-2007: Added vertical option.
  * 01-Nov-2000: Allocation checks
  * 10-May-1997: Compatible with xscreensaver
  * 16-Apr-1997: -neighbors 3, 9 (not sound mathematically), 12, and 8 added
@@ -73,21 +74,27 @@ static const char sccsid[] = "@(#)demon.c	5.00 2000/11/01 xlockmore";
  * neighbors of 0 randomizes it between 3, 4, 6, 8, 9, and 12.
  */
 #define DEF_NEIGHBORS  "0"      /* choose random value */
+#define DEF_VERTICAL "False"
 
 static int  neighbors;
+static Bool vertical;
 
 static XrmOptionDescRec opts[] =
 {
-	{(char *) "-neighbors", (char *) ".demon.neighbors", XrmoptionSepArg, (caddr_t) NULL}
-};
+	{(char *) "-neighbors", (char *) ".demon.neighbors", XrmoptionSepArg, (caddr_t) NULL},
+	{(char *) "-vertical", (char *) ".demon.vertical", XrmoptionNoArg, (caddr_t) "on"},
+	{(char *) "+vertical", (char *) ".demon.vertical", XrmoptionNoArg, (caddr_t) "off"}
 
+};
 static argtype vars[] =
 {
-	{(void *) & neighbors, (char *) "neighbors", (char *) "Neighbors", (char *) DEF_NEIGHBORS, t_Int}
+	{(void *) & neighbors, (char *) "neighbors", (char *) "Neighbors", (char *) DEF_NEIGHBORS, t_Int},
+	{(void *) & vertical, (char *) "vertical", (char *) "Vertical", (char *) DEF_VERTICAL, t_Bool}
 };
 static OptionStruct desc[] =
 {
-	{(char *) "-neighbors num", (char *) "squares 4 or 8, hexagons 6, triangles 3, 9 or 12"}
+	{(char *) "-neighbors num", (char *) "squares 4 or 8, hexagons 6, triangles 3, 9 or 12"},
+	{(char *) "-/+vertical", (char *) "change orientation for hexagons and triangles"}
 };
 
 ModeSpecOpt demon_opts =
@@ -120,6 +127,7 @@ typedef struct _CellList {
 } CellList;
 
 typedef struct {
+	Bool        vertical;
 	int         generation;
 	int         xs, ys;
 	int         xb, yb;
@@ -160,8 +168,8 @@ drawcell(ModeInfo * mi, int col, int row, unsigned char state)
 		gc = MI_GC(mi);
 	} else if (MI_NPIXELS(mi) >= NUMSTIPPLES) {
 		XSetForeground(MI_DISPLAY(mi), MI_GC(mi),
-			   MI_PIXEL(mi, (((int) state - 1) * MI_NPIXELS(mi) /
-					 (dp->states - 1)) % MI_NPIXELS(mi)));
+			MI_PIXEL(mi, (((int) state - 1) * MI_NPIXELS(mi) /
+			(dp->states - 1)) % MI_NPIXELS(mi)));
 		gc = MI_GC(mi);
 	} else {
 		XGCValues   gcv;
@@ -174,36 +182,58 @@ drawcell(ModeInfo * mi, int col, int row, unsigned char state)
 		gc = dp->stippledGC;
 	}
 	if (dp->neighbors == 6) {
-		int         ccol = 2 * col + !(row & 1), crow = 2 * row;
+		int ccol = 2 * col + !(row & 1), crow = 2 * row;
 
-		dp->shape.hexagon[0].x = dp->xb + ccol * dp->xs;
-		dp->shape.hexagon[0].y = dp->yb + crow * dp->ys;
+		if (dp->vertical) {
+			dp->shape.hexagon[0].x = dp->xb + ccol * dp->xs;
+			dp->shape.hexagon[0].y = dp->yb + crow * dp->ys;
+		} else {
+			dp->shape.hexagon[0].y = dp->xb + ccol * dp->xs;
+			dp->shape.hexagon[0].x = dp->yb + crow * dp->ys;
+		}
 		if (dp->xs == 1 && dp->ys == 1)
-			XDrawPoint(MI_DISPLAY(mi), MI_WINDOW(mi),
-				       gc, dp->shape.hexagon[0].x, dp->shape.hexagon[0].y);
+			XDrawPoint(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
+				dp->shape.hexagon[0].x,
+				dp->shape.hexagon[0].y);
 		else
 			XFillPolygon(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
-			    dp->shape.hexagon, 6, Convex, CoordModePrevious);
+				dp->shape.hexagon, 6,
+				Convex, CoordModePrevious);
 	} else if (dp->neighbors == 4 || dp->neighbors == 8) {
 		XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
-		dp->xb + dp->xs * col, dp->yb + dp->ys * row,
-		dp->xs - (dp->xs > 3), dp->ys - (dp->ys > 3));
+			dp->xb + dp->xs * col, dp->yb + dp->ys * row,
+			dp->xs - (dp->xs > 3), dp->ys - (dp->ys > 3));
 	} else {		/* TRI */
-		int         orient = (col + row) % 2;	/* O left 1 right */
+		int orient = (col + row) % 2;	/* O left 1 right */
+		Bool small = (dp->xs <= 3 || dp->ys <= 3);
 
-		dp->shape.triangle[orient][0].x = dp->xb + col * dp->xs;
-		dp->shape.triangle[orient][0].y = dp->yb + row * dp->ys;
-		if (dp->xs <= 3 || dp->ys <= 3)
-			XDrawPoint(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
-			((orient) ? -1 : 1) + dp->shape.triangle[orient][0].x,
-				       dp->shape.triangle[orient][0].y);
-		else {
-			if (orient)
-				dp->shape.triangle[orient][0].x += (dp->xs / 2 - 1);
+		if (dp->vertical) {
+			dp->shape.triangle[orient][0].x = dp->xb + col * dp->xs;
+			dp->shape.triangle[orient][0].y = dp->yb + row * dp->ys;
+			if (small)
+				dp->shape.triangle[orient][0].x +=
+					((orient) ? -1 : 1);
 			else
-				dp->shape.triangle[orient][0].x -= (dp->xs / 2 - 1);
+				dp->shape.triangle[orient][0].x +=
+					(dp->xs / 2  - 1) * ((orient) ? 1 : -1);
+		} else {
+			dp->shape.triangle[orient][0].y = dp->xb + col * dp->xs;
+			dp->shape.triangle[orient][0].x = dp->yb + row * dp->ys;
+			if (small)
+				dp->shape.triangle[orient][0].y +=
+					((orient) ? -1 : 1);
+			else
+				dp->shape.triangle[orient][0].y +=
+					(dp->xs / 2  - 1) * ((orient) ? 1 : -1);
+		}
+		if (small)
+			XDrawPoint(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
+				dp->shape.triangle[orient][0].x,
+				dp->shape.triangle[orient][0].y);
+		else {
 			XFillPolygon(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
-				     dp->shape.triangle[orient], 3, Convex, CoordModePrevious);
+				dp->shape.triangle[orient], 3,
+				Convex, CoordModePrevious);
 
 		}
 	}
@@ -344,14 +374,21 @@ draw_state(ModeInfo * mi, int state)
 			col = current->pt.x;
 			row = current->pt.y;
 			ccol = 2 * col + !(row & 1), crow = 2 * row;
-			dp->shape.hexagon[0].x = dp->xb + ccol * dp->xs;
-			dp->shape.hexagon[0].y = dp->yb + crow * dp->ys;
+			if (dp->vertical) {
+				dp->shape.hexagon[0].x = dp->xb + ccol * dp->xs;
+				dp->shape.hexagon[0].y = dp->yb + crow * dp->ys;
+			} else {
+				dp->shape.hexagon[0].y = dp->xb + ccol * dp->xs;
+				dp->shape.hexagon[0].x = dp->yb + crow * dp->ys;
+			}
 			if (dp->xs == 1 && dp->ys == 1)
-				XDrawPoint(MI_DISPLAY(mi), MI_WINDOW(mi),
-					       gc, dp->shape.hexagon[0].x, dp->shape.hexagon[0].y);
+				XDrawPoint(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
+					dp->shape.hexagon[0].x,
+					dp->shape.hexagon[0].y);
 			else
 				XFillPolygon(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
-					     dp->shape.hexagon, 6, Convex, CoordModePrevious);
+					dp->shape.hexagon, 6,
+					Convex, CoordModePrevious);
 			current = current->next;
 		}
 	} else if (dp->neighbors == 4 || dp->neighbors == 8) {
@@ -377,6 +414,8 @@ draw_state(ModeInfo * mi, int state)
 		/* Free up rects list and the appropriate part of the cellList */
 		free(rects);
 	} else {		/* TRI */
+		Bool small = (dp->xs <= 3 || dp->ys <= 3);
+
 		current = dp->cellList[state];
 		while (current) {
 			int         col, row, orient;
@@ -384,19 +423,33 @@ draw_state(ModeInfo * mi, int state)
 			col = current->pt.x;
 			row = current->pt.y;
 			orient = (col + row) % 2;	/* O left 1 right */
-			dp->shape.triangle[orient][0].x = dp->xb + col * dp->xs;
-			dp->shape.triangle[orient][0].y = dp->yb + row * dp->ys;
-			if (dp->xs <= 3 || dp->ys <= 3)
-				XDrawPoint(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
-					       ((orient) ? -1 : 1) + dp->shape.triangle[orient][0].x,
-				      dp->shape.triangle[orient][0].y);
-			else {
-				if (orient)
-					dp->shape.triangle[orient][0].x += (dp->xs / 2 - 1);
+			if (dp->vertical) {
+				dp->shape.triangle[orient][0].x = dp->xb + col * dp->xs;
+				dp->shape.triangle[orient][0].y = dp->yb + row * dp->ys;
+				if (small)
+					dp->shape.triangle[orient][0].x +=
+						((orient) ? -1 : 1);
 				else
-					dp->shape.triangle[orient][0].x -= (dp->xs / 2 - 1);
+					dp->shape.triangle[orient][0].x +=
+						(dp->xs / 2  - 1) * ((orient) ? 1 : -1);
+			} else {
+				dp->shape.triangle[orient][0].y = dp->xb + col * dp->xs;
+				dp->shape.triangle[orient][0].x = dp->yb + row * dp->ys;
+				if (small)
+					dp->shape.triangle[orient][0].y +=
+						((orient) ? -1 : 1);
+				else
+					dp->shape.triangle[orient][0].y +=
+						(dp->xs / 2  - 1) * ((orient) ? 1 : -1);
+			}
+			if (small)
+				XDrawPoint(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
+					dp->shape.triangle[orient][0].x,
+					dp->shape.triangle[orient][0].y);
+			else {
 				XFillPolygon(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
-					     dp->shape.triangle[orient], 3, Convex, CoordModePrevious);
+					dp->shape.triangle[orient], 3,
+					Convex, CoordModePrevious);
 			}
 			current = current->next;
 		}
@@ -489,12 +542,20 @@ init_demon(ModeInfo * mi)
 	}
 
 	dp->state = 0;
-
+	if (MI_IS_FULLRANDOM(mi)) {
+		dp->vertical = (Bool) (LRAND() & 1);
+	} else {
+		dp->vertical = vertical;
+	}
 	dp->width = MI_WIDTH(mi);
 	dp->height = MI_HEIGHT(mi);
 	if (dp->neighbors == 6) {
-		int         nccols, ncrows, i;
+		int nccols, ncrows, i;
 
+		if (!dp->vertical) {
+			dp->height = MI_WIDTH(mi);
+			dp->width = MI_HEIGHT(mi);
+		}
 		if (dp->width < 8)
 			dp->width = 8;
 		if (dp->height < 8)
@@ -516,10 +577,22 @@ init_demon(ModeInfo * mi)
 		dp->ncols = nccols / 2;
 		dp->nrows = 2 * (ncrows / 4);
 		dp->xb = (dp->width - dp->xs * nccols) / 2 + dp->xs / 2;
-		dp->yb = (dp->height - dp->ys * (ncrows / 2) * 2) / 2 + dp->ys / 4;
+		dp->yb = (dp->height - dp->ys * (ncrows / 2) * 2) / 2 +
+			dp->ys / 3;
 		for (i = 0; i < 6; i++) {
-			dp->shape.hexagon[i].x = (dp->xs - 1) * hexagonUnit[i].x;
-			dp->shape.hexagon[i].y = ((dp->ys - 1) * hexagonUnit[i].y / 2) * 4 / 3;
+			if (dp->vertical) {
+				dp->shape.hexagon[i].x =
+					(dp->xs - 1) * hexagonUnit[i].x;
+				dp->shape.hexagon[i].y =
+					((dp->ys - 1) * hexagonUnit[i].y /
+					2) * 4 / 3;
+			} else {
+				dp->shape.hexagon[i].y =
+					(dp->xs - 1) * hexagonUnit[i].x;
+				dp->shape.hexagon[i].x =
+					((dp->ys - 1) * hexagonUnit[i].y /
+					2) * 4 / 3;
+			}
 		}
 	} else if (dp->neighbors == 4 || dp->neighbors == 8) {
 		if (size < -MINSIZE)
@@ -539,8 +612,12 @@ init_demon(ModeInfo * mi)
 		dp->xb = (dp->width - dp->xs * dp->ncols) / 2;
 		dp->yb = (dp->height - dp->ys * dp->nrows) / 2;
 	} else {		/* TRI */
-		int         orient, i;
+		int orient, i;
 
+		if (!dp->vertical) {
+			dp->height = MI_WIDTH(mi);
+			dp->width = MI_HEIGHT(mi);
+		}
 		if (dp->width < 2)
 			dp->width = 2;
 		if (dp->height < 2)
@@ -563,10 +640,17 @@ init_demon(ModeInfo * mi)
 		dp->yb = (dp->height - dp->ys * dp->nrows) / 2 + dp->ys / 2;
 		for (orient = 0; orient < 2; orient++) {
 			for (i = 0; i < 3; i++) {
-				dp->shape.triangle[orient][i].x =
-					(dp->xs - 2) * triangleUnit[orient][i].x;
-				dp->shape.triangle[orient][i].y =
-					(dp->ys - 2) * triangleUnit[orient][i].y;
+				if (dp->vertical) {
+					dp->shape.triangle[orient][i].x =
+						(dp->xs - 2) * triangleUnit[orient][i].x;
+					dp->shape.triangle[orient][i].y =
+						(dp->ys - 2) * triangleUnit[orient][i].y;
+				} else {
+					dp->shape.triangle[orient][i].y =
+						(dp->xs - 2) * triangleUnit[orient][i].x;
+					dp->shape.triangle[orient][i].x =
+						(dp->ys - 2) * triangleUnit[orient][i].y;
+				}
 			}
 		}
 	}
