@@ -2,7 +2,7 @@
 /* dilemma --- Lloyd's Prisoner's Dilemma Simulation */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)dilemma.c	5.00 2000/11/01 xlockmore";
+static const char sccsid[] = "@(#)dilemma.c	5.24 2007/01/18 xlockmore";
 
 #endif
 
@@ -22,6 +22,7 @@ static const char sccsid[] = "@(#)dilemma.c	5.00 2000/11/01 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
+ * 18-Jan-2007: Added vertical option.
  * 01-Nov-2000: Allocation checks
  * 20-Oct-1997: Computing Bouts of the Prisoner's Dilemma by Alun L. Lloyd
  *              Scientific American Magazine June 1995
@@ -91,29 +92,35 @@ static const char sccsid[] = "@(#)dilemma.c	5.00 2000/11/01 xlockmore";
 #define DEF_NEIGHBORS  "0"      /* choose random value */
 #define DEF_BONUS  "1.85"
 #define DEF_CONSCIOUS  "True"
+#define DEF_VERTICAL "False"
 
 static int  neighbors;
 static float bonus;
 static Bool conscious;
+static Bool vertical;
 
 static XrmOptionDescRec opts[] =
 {
 	{(char *) "-neighbors", (char *) ".dilemma.neighbors", XrmoptionSepArg, (caddr_t) NULL},
 	{(char *) "-bonus", (char *) ".dilemma.bonus", XrmoptionSepArg, (caddr_t) NULL},
 	{(char *) "-conscious", (char *) ".dilemma.conscious", XrmoptionNoArg, (caddr_t) "on"},
-	{(char *) "+conscious", (char *) ".dilemma.conscious", XrmoptionNoArg, (caddr_t) "off"}
+	{(char *) "+conscious", (char *) ".dilemma.conscious", XrmoptionNoArg, (caddr_t) "off"},
+	{(char *) "-vertical", (char *) ".dilemma.vertical", XrmoptionNoArg, (caddr_t) "on"},
+	{(char *) "+vertical", (char *) ".dilemma.vertical", XrmoptionNoArg, (caddr_t) "off"}
 };
 static argtype vars[] =
 {
-	{(caddr_t *) & neighbors, (char *) "neighbors", (char *) "Neighbors", (char *) DEF_NEIGHBORS, t_Int},
-	{(caddr_t *) & bonus, (char *) "bonus", (char *) "Bonus", (char *) DEF_BONUS, t_Float},
-   {(caddr_t *) & conscious, (char *) "conscious", (char *) "Conscious", (char *) DEF_CONSCIOUS, t_Bool}
+	{(void *) & neighbors, (char *) "neighbors", (char *) "Neighbors", (char *) DEF_NEIGHBORS, t_Int},
+	{(void *) & bonus, (char *) "bonus", (char *) "Bonus", (char *) DEF_BONUS, t_Float},
+	{(void *) & conscious, (char *) "conscious", (char *) "Conscious", (char *) DEF_CONSCIOUS, t_Bool},
+	{(void *) & vertical, (char *) "vertical", (char *) "Vertical", (char *) DEF_VERTICAL, t_Bool}
 };
 static OptionStruct desc[] =
 {
 	{(char *) "-neighbors num", (char *) "squares 4 or 8, hexagons 6, triangles 3, 9 or 12"},
 	{(char *) "-bonus value", (char *) "bonus for cheating... between 1.0 and 4.0"},
-	{(char *) "-/+conscious", (char *) "turn on/off self-awareness"}
+	{(char *) "-/+conscious", (char *) "turn on/off self-awareness"},
+	{(char *) "-/+vertical", (char *) "change orientation for hexagons and triangles"}
 };
 
 ModeSpecOpt dilemma_opts =
@@ -163,6 +170,7 @@ typedef struct _CellList {
 } CellList;
 
 typedef struct {
+	Bool        vertical;
 	int         defectors;	/* portion of defectors */
 	float       pm[2][2];	/* payoff matrix */
 	unsigned long colors[2][2];
@@ -196,98 +204,166 @@ static int  icon_width, icon_height;
 
 static void
 drawcell(ModeInfo * mi, int col, int row, unsigned long color, int bitmap,
-	 Bool firstChange)
+		Bool firstChange)
 {
 	Display    *display = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
 	GC          gc = MI_GC(mi);
 	dilemmastruct *dp = &dilemmas[MI_SCREEN(mi)];
 	unsigned long colour = (MI_NPIXELS(mi) >= COLORS) ?
-	color : MI_WHITE_PIXEL(mi);
+		color : MI_WHITE_PIXEL(mi);
 
 	XSetForeground(display, gc, colour);
 	if (dp->neighbors == 6) {
-		int         ccol = 2 * col + !(row & 1), crow = 2 * row;
+		int ccol = 2 * col + !(row & 1), crow = 2 * row;
 
-		dp->shape.hexagon[0].x = dp->xb + ccol * dp->xs;
-		dp->shape.hexagon[0].y = dp->yb + crow * dp->ys;
+		if (dp->vertical) {
+			dp->shape.hexagon[0].x = dp->xb + ccol * dp->xs;
+			dp->shape.hexagon[0].y = dp->yb + crow * dp->ys;
+		} else {
+			dp->shape.hexagon[0].y = dp->xb + ccol * dp->xs;
+			dp->shape.hexagon[0].x = dp->yb + crow * dp->ys;
+		}
 		if (dp->xs == 1 && dp->ys == 1)
 			XDrawPoint(display, window, gc,
-			dp->shape.hexagon[0].x, dp->shape.hexagon[0].y);
+				dp->shape.hexagon[0].x,
+				dp->shape.hexagon[0].y);
 		else if (bitmap == BITMAPS - 1)
 			XFillPolygon(display, window, gc,
-			    dp->shape.hexagon, 6, Convex, CoordModePrevious);
+				dp->shape.hexagon, 6,
+				Convex, CoordModePrevious);
 		else {
+			int ix = 0, iy = 0, sx, sy;
+
 			if (firstChange) {
 				XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 				XFillPolygon(display, window, gc,
-					     dp->shape.hexagon, 6, Convex, CoordModePrevious);
+					dp->shape.hexagon, 6,
+					Convex, CoordModePrevious);
 				XSetForeground(display, gc, colour);
 			}
+			if (dp->vertical) {
+				dp->shape.hexagon[0].x -= dp->xs;
+				dp->shape.hexagon[0].y += dp->ys / 4;
+				sx = 2 * dp->xs - 6;
+				sy = 2 * dp->ys - 2;
+				if (dp->xs <= 6 || dp->ys <= 2) {
+					ix = 3;
+					iy = 1;
+				} else
+					ix = 5;
+			} else {
+				dp->shape.hexagon[0].y -= dp->xs;
+				dp->shape.hexagon[0].x += dp->ys / 4;
+				sy = 2 * dp->xs - 6;
+				sx = 2 * dp->ys - 2;
+				if (dp->xs <= 6 || dp->ys <= 2) {
+					iy = 3;
+					ix = 1;
+				} else
+					iy = 5;
+			}
 			if (dp->xs <= 6 || dp->ys <= 2)
 				XFillRectangle(display, window, gc,
-				     dp->shape.hexagon[0].x - 3 * dp->xs / 4,
-					       dp->shape.hexagon[0].y + dp->ys / 4, dp->xs, dp->ys);
+					dp->shape.hexagon[0].x + ix,
+					dp->shape.hexagon[0].y + iy,
+					dp->xs, dp->ys);
 			else
 				XFillArc(display, window, gc,
-				     dp->xb + dp->xs * ccol - 3 * dp->xs / 4,
-					 dp->yb + dp->ys * crow + dp->ys / 4,
-					 2 * dp->xs - 6, 2 * dp->ys - 2,
-					 0, 23040);
+					dp->shape.hexagon[0].x + ix,
+					dp->shape.hexagon[0].y + iy,
+					sy, sy,
+					0, 23040);
 		}
 	} else if (dp->neighbors == 4 || dp->neighbors == 8) {
 		if (dp->pixelmode) {
 			if (bitmap == BITMAPS - 1 || (dp->xs <= 2 || dp->ys <= 2))
 				XFillRectangle(display, window, gc,
-				dp->xb + dp->xs * col, dp->yb + dp->ys * row,
-				dp->xs - (dp->xs > 3), dp->ys - (dp->ys > 3));
+					dp->xb + dp->xs * col,
+					dp->yb + dp->ys * row,
+					dp->xs - (dp->xs > 3),
+					dp->ys - (dp->ys > 3));
 			else {
 				if (firstChange) {
-					XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
+					XSetForeground(display, gc,
+						MI_BLACK_PIXEL(mi));
 					XFillRectangle(display, window, gc,
-						       dp->xb + dp->xs * col, dp->yb + dp->ys * row,
-						       dp->xs, dp->ys);
+						dp->xb + dp->xs * col,
+						dp->yb + dp->ys * row,
+						dp->xs, dp->ys);
 					XSetForeground(display, gc, colour);
 				}
 				XFillArc(display, window, gc,
-					 dp->xb + dp->xs * col, dp->yb + dp->ys * row,
-					 dp->xs - 1, dp->ys - 1,
-					 0, 23040);
+					dp->xb + dp->xs * col,
+					dp->yb + dp->ys * row,
+					dp->xs - 1, dp->ys - 1,
+					0, 23040);
 			}
 		} else
 			(void) XPutImage(display, window, gc,
-					 &logo[bitmap], 0, 0,
+				&logo[bitmap], 0, 0,
 				dp->xb + dp->xs * col, dp->yb + dp->ys * row,
-					 icon_width, icon_height);
+				icon_width, icon_height);
 	} else {		/* TRI */
-		int         orient = (col + row) % 2;	/* O left 1 right */
+		int orient = (col + row) % 2;	/* O left 1 right */
+		Bool small = (dp->xs <= 3 || dp->ys <= 3);
+		int ix = 0, iy = 0;
 
-		dp->shape.triangle[orient][0].x = dp->xb + col * dp->xs;
-		dp->shape.triangle[orient][0].y = dp->yb + row * dp->ys;
-		if (dp->xs <= 3 || dp->ys <= 3)
-			XDrawPoint(display, window, gc,
-			((orient) ? -1 : 1) + dp->shape.triangle[orient][0].x,
-				       dp->shape.triangle[orient][0].y);
-		else {
-			if (orient)
-				dp->shape.triangle[orient][0].x += (dp->xs / 2 - 1);
+		if (dp->vertical) {
+			dp->shape.triangle[orient][0].x = dp->xb + col * dp->xs;
+			dp->shape.triangle[orient][0].y = dp->yb + row * dp->ys;
+			if (small)
+				dp->shape.triangle[orient][0].x +=
+					((orient) ? -1 : 1);
 			else
-				dp->shape.triangle[orient][0].x -= (dp->xs / 2 - 1);
+				dp->shape.triangle[orient][0].x +=
+					(dp->xs / 2  - 1) * ((orient) ? 1 : -1);
+			ix = ((orient) ? -dp->xs / 2 : dp->xs / 2);
+		} else {
+			dp->shape.triangle[orient][0].y = dp->xb + col * dp->xs;
+			dp->shape.triangle[orient][0].x = dp->yb + row * dp->ys;
+			if (small)
+				dp->shape.triangle[orient][0].y +=
+					((orient) ? -1 : 1);
+			else
+				dp->shape.triangle[orient][0].y +=
+					(dp->xs / 2  - 1) * ((orient) ? 1 : -1);
+			iy = ((orient) ? -dp->xs / 2 : dp->xs / 2);
+		}
+		if (small)
+			XDrawPoint(display, window, gc,
+				dp->shape.triangle[orient][0].x,
+				dp->shape.triangle[orient][0].y);
+		else {
 			if (bitmap == BITMAPS - 1)
 				XFillPolygon(display, window, gc,
-					     dp->shape.triangle[orient], 3, Convex, CoordModePrevious);
+					dp->shape.triangle[orient], 3,
+					Convex, CoordModePrevious);
 			else {
 				if (firstChange) {
-					XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
+					XSetForeground(display, gc,
+						MI_BLACK_PIXEL(mi));
 					XFillPolygon(display, window, gc,
-						     dp->shape.triangle[orient], 3, Convex, CoordModePrevious);
+						dp->shape.triangle[orient], 3,
+						Convex, CoordModePrevious);
 					XSetForeground(display, gc, colour);
 				}
+				if (dp->vertical) {
+					dp->shape.triangle[orient][0].x += -4 * dp->xs / 5 +
+						((orient) ? dp->xs / 3 : 3 * dp->xs / 5);
+					dp->shape.triangle[orient][0].y += -dp->ys / 2 + 1;
+					ix = ((orient) ? -dp->xs / 2 : dp->xs / 2);
+				} else {
+					dp->shape.triangle[orient][0].y += -4 * dp->xs / 5 +
+						((orient) ? dp->xs / 3 : 3 * dp->xs / 5);
+					dp->shape.triangle[orient][0].x += -dp->ys / 2 + 1;
+					iy = ((orient) ? -dp->xs / 2 : dp->xs / 2);
+				}
 				XFillArc(display, window, gc,
-				     dp->xb + dp->xs * col - 4 * dp->xs / 5 +
-				    ((orient) ? dp->xs / 3 : 3 * dp->xs / 5),
-					 dp->yb + dp->ys * row - dp->ys / 2 + 1, dp->ys - 3, dp->ys - 3,
-					 0, 23040);
+					dp->shape.triangle[orient][0].x + ix,
+					dp->shape.triangle[orient][0].y + iy,
+					dp->ys - 3, dp->ys - 3,
+					0, 23040);
 			}
 		}
 	}
@@ -621,7 +697,7 @@ draw_state(ModeInfo * mi, int state)
 
 		drawcell(mi, col, row,
 		       dp->colors[(int) dp->sn[colrow]][(int) dp->s[colrow]],
-			 dp->sn[colrow], 1);
+			 dp->sn[colrow], False);
 		if (dp->s[colrow] && !dp->sn[colrow])
 			dp->defectors--;
 		if (!dp->s[colrow] && dp->sn[colrow])
@@ -662,6 +738,11 @@ init_dilemma(ModeInfo * mi)
 			logo[i].bytes_per_line = (icon_width + 7) / 8;
 		}
 	}
+	if (MI_IS_FULLRANDOM(mi)) {
+		dp->vertical = (Bool) (LRAND() & 1);
+	} else {
+		dp->vertical = vertical;
+	}
 	dp->width = MI_WIDTH(mi);
 	dp->height = MI_HEIGHT(mi);
 
@@ -682,8 +763,12 @@ init_dilemma(ModeInfo * mi)
 	}
 
 	if (dp->neighbors == 6) {
-		int         nccols, ncrows, sides;
+		int nccols, ncrows, sides;
 
+		if (!dp->vertical) {
+			dp->height = MI_WIDTH(mi);
+			dp->width = MI_HEIGHT(mi);
+		}
 		if (dp->width < 2)
 			dp->width = 2;
 		if (dp->height < 4)
@@ -706,11 +791,22 @@ init_dilemma(ModeInfo * mi)
 		dp->ncols = nccols / 2;
 		dp->nrows = 2 * (ncrows / 4);
 		dp->xb = (dp->width - dp->xs * nccols) / 2 + dp->xs / 2;
-		dp->yb = (dp->height - dp->ys * (ncrows / 2) * 2) / 2 + dp->ys - 2;
+		dp->yb = (dp->height - dp->ys * (ncrows / 2) * 2) / 2 +
+			dp->ys - 2;
 		for (sides = 0; sides < 6; sides++) {
-			dp->shape.hexagon[sides].x = (dp->xs - 1) * hexagonUnit[sides].x;
-			dp->shape.hexagon[sides].y =
-				((dp->ys - 1) * hexagonUnit[sides].y / 2) * 4 / 3;
+			if (dp->vertical) {
+				dp->shape.hexagon[sides].x =
+					(dp->xs - 1) * hexagonUnit[sides].x;
+				dp->shape.hexagon[sides].y =
+					((dp->ys - 1) * hexagonUnit[sides].y /
+					2) * 4 / 3;
+			} else {
+				dp->shape.hexagon[sides].y =
+					(dp->xs - 1) * hexagonUnit[sides].x;
+				dp->shape.hexagon[sides].x =
+					((dp->ys - 1) * hexagonUnit[sides].y /
+					2) * 4 / 3;
+			}
 		}
 	} else if (dp->neighbors == 4 || dp->neighbors == 8) {
 		if (dp->width < 2)
@@ -746,8 +842,12 @@ init_dilemma(ModeInfo * mi)
 		dp->xb = (dp->width - dp->xs * dp->ncols) / 2;
 		dp->yb = (dp->height - dp->ys * dp->nrows) / 2;
 	} else {		/* TRI */
-		int         orient, sides;
+		int orient, sides;
 
+		if (!dp->vertical) {
+			dp->height = MI_WIDTH(mi);
+			dp->width = MI_HEIGHT(mi);
+		}
 		if (dp->width < 2)
 			dp->width = 2;
 		if (dp->height < 2)
@@ -770,11 +870,18 @@ init_dilemma(ModeInfo * mi)
 		dp->xb = (dp->width - dp->xs * dp->ncols) / 2 + dp->xs / 2;
 		dp->yb = (dp->height - dp->ys * dp->nrows) / 2 + dp->ys / 2;
 		for (orient = 0; orient < 2; orient++) {
-			for (sides = 0; sides < 3; sides++) {
-				dp->shape.triangle[orient][sides].x =
-					(dp->xs - 2) * triangleUnit[orient][sides].x;
-				dp->shape.triangle[orient][sides].y =
-					(dp->ys - 2) * triangleUnit[orient][sides].y;
+			for (i = 0; i < 3; i++) {
+				if (dp->vertical) {
+					dp->shape.triangle[orient][i].x =
+						(dp->xs - 2) * triangleUnit[orient][i].x;
+					dp->shape.triangle[orient][i].y =
+						(dp->ys - 2) * triangleUnit[orient][i].y;
+				} else {
+					dp->shape.triangle[orient][i].y =
+						(dp->xs - 2) * triangleUnit[orient][i].x;
+					dp->shape.triangle[orient][i].x =
+						(dp->ys - 2) * triangleUnit[orient][i].y;
+				}
 			}
 		}
 	}
@@ -916,7 +1023,7 @@ draw_dilemma(ModeInfo * mi)
 		for (i = 0; i < REDRAWSTEP; i++) {
 			drawcell(mi, dp->redrawpos % dp->ncols, dp->redrawpos / dp->ncols,
 				 dp->colors[(int) dp->sn[dp->redrawpos]][(int) dp->s[dp->redrawpos]],
-				 dp->sn[dp->redrawpos], 1);
+				 dp->sn[dp->redrawpos], False);
 			if (++(dp->redrawpos) >= dp->npositions) {
 				dp->redrawing = 0;
 				break;
