@@ -2,7 +2,7 @@
 /* loop --- Chris Langton's self-producing loops */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)loop.c	5.12 2004/03/15 xlockmore";
+static const char sccsid[] = "@(#)loop.c	5.24 2007/01/18 xlockmore";
 
 #endif
 
@@ -22,8 +22,7 @@ static const char sccsid[] = "@(#)loop.c	5.12 2004/03/15 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
-
- Proposed (some changes for this present but not functional yet):
+ * 18-Jan-2007: Added vertical option.
  * 10-Apr-2004: EvoLoopS - Evolving SDSR Loop Simulator idea stolen
  *              from Hiroki Sayama <sayama AT is.s.u-tokyo.ac.jp>.
  *              -evolve option
@@ -116,20 +115,26 @@ ping.
  * neighbors of 0 randomizes between 4 and 6.
  */
 #define DEF_NEIGHBORS  "0"      /* choose random value */
+#define DEF_LANGTON  "False"    /* neighbors 4 or 6 */
 #define DEF_EVOLVE  "False"     /* neighbors 4 only */
 #define DEF_DISSOLVE  "False"   /* neighbors 4 only and no evolve */
 #define DEF_SHEATH  "False"     /* neighbors 4 only and no evolve */
 #define DEF_WRAP  "True"
+#define DEF_VERTICAL  "False"
 
 static int  neighbors;
+static Bool langton;
 static Bool evolve;
 static Bool dissolve;
 static Bool sheath;
 static Bool wrap;
+static Bool vertical;
 
 static XrmOptionDescRec opts[] =
 {
 	{(char *) "-neighbors", (char *) ".loop.neighbors", XrmoptionSepArg, (caddr_t) NULL},
+	{(char *) "-langton", (char *) ".loop.langton", XrmoptionNoArg, (caddr_t) "on"},
+	{(char *) "+langton", (char *) ".loop.langton", XrmoptionNoArg, (caddr_t) "off"},
 	{(char *) "-evolve", (char *) ".loop.evolve", XrmoptionNoArg, (caddr_t) "on"},
 	{(char *) "+evolve", (char *) ".loop.evolve", XrmoptionNoArg, (caddr_t) "off"},
 	{(char *) "-dissolve", (char *) ".loop.dissolve", XrmoptionNoArg, (caddr_t) "on"},
@@ -137,25 +142,31 @@ static XrmOptionDescRec opts[] =
 	{(char *) "-sheath", (char *) ".loop.sheath", XrmoptionNoArg, (caddr_t) "on"},
 	{(char *) "+sheath", (char *) ".loop.sheath", XrmoptionNoArg, (caddr_t) "off"},
 	{(char *) "-wrap", (char *) ".loop.wrap", XrmoptionNoArg, (caddr_t) "on"},
-	{(char *) "+wrap", (char *) ".loop.wrap", XrmoptionNoArg, (caddr_t) "off"}
+	{(char *) "+wrap", (char *) ".loop.wrap", XrmoptionNoArg, (caddr_t) "off"},
+	{(char *) "-vertical", (char *) ".loop.vertical", XrmoptionNoArg, (caddr_t) "on"},
+	{(char *) "+vertical", (char *) ".loop.vertical", XrmoptionNoArg, (caddr_t) "off"}
 };
 
 static argtype vars[] =
 {
-	{(caddr_t *) & neighbors, (char *) "neighbors", (char *) "Neighbors", (char *) DEF_NEIGHBORS, t_Int},
-	{(caddr_t *) & evolve, (char *) "evolve", (char *) "Evolve", (char *) DEF_EVOLVE, t_Bool},
-	{(caddr_t *) & dissolve, (char *) "dissolve", (char *) "Dissolve", (char *) DEF_DISSOLVE, t_Bool},
-	{(caddr_t *) & sheath, (char *) "sheath", (char *) "Extension", (char *) DEF_SHEATH, t_Bool},
-	{(caddr_t *) & wrap, (char *) "wrap", (char *) "Wrap", (char *) DEF_WRAP, t_Bool}
+	{(void *) & neighbors, (char *) "neighbors", (char *) "Neighbors", (char *) DEF_NEIGHBORS, t_Int},
+	{(void *) & langton, (char *) "langton", (char *) "Langton", (char *) DEF_LANGTON, t_Bool},
+	{(void *) & evolve, (char *) "evolve", (char *) "Evolve", (char *) DEF_EVOLVE, t_Bool},
+	{(void *) & dissolve, (char *) "dissolve", (char *) "Dissolve", (char *) DEF_DISSOLVE, t_Bool},
+	{(void *) & sheath, (char *) "sheath", (char *) "Extension", (char *) DEF_SHEATH, t_Bool},
+	{(void *) & wrap, (char *) "wrap", (char *) "Wrap", (char *) DEF_WRAP, t_Bool},
+	{(void *) & vertical, (char *) "vertical", (char *) "Vertical", (char *) DEF_VERTICAL, t_Bool}
 };
 
 static OptionStruct desc[] =
 {
 	{(char *) "-neighbors num", (char *) "squares 4 or hexagons 6"},
-	{(char *) "-/+evolve", (char *) "turn on/off evolving loops"},
+	{(char *) "-/+langton", (char *) "turn on/off Langton's Loops"},
+	{(char *) "-/+evolve", (char *) "turn on/off Evolving Loops"},
 	{(char *) "-/+dissolve", (char *) "turn on/off structurally dissolvable loops"},
 	{(char *) "-/+sheath", (char *) "turn on/off sheath extension"},
-	{(char *) "-/+wrap", (char *) "turn on/off wrapping"}
+	{(char *) "-/+wrap", (char *) "turn on/off wrapping"},
+	{(char *) "-/+vertical", (char *) "change orientation for hexagons"}
 };
 
 ModeSpecOpt loop_opts =
@@ -183,7 +194,7 @@ static int  local_neighbors = 0;
    -delay 1 -count 170 -neighbors 6  # divisions starts
    540 first cell collision
    1111 mutant being born from 2 parents
-   1156 mutant born 
+   1156 mutant born
  */
 #define DELAYDEBUGLOOP 10
 #endif
@@ -250,7 +261,7 @@ typedef struct {
 	int         mincol, minrow, maxcol, maxrow;
 	int         width, height;
 	int         redrawing, redrawpos;
-	Bool        dead, clockwise, wrap;
+	Bool        dead, clockwise, wrap, vertical;
 	unsigned char *newcells, *oldcells;
 	int         ncells[EXT_STATES];
 	CellList   *cellList[EXT_STATES];
@@ -264,11 +275,11 @@ typedef struct {
 
 static loopstruct *loops = (loopstruct *) NULL;
 
-static int M8[]={1, 8, 64, 512, 4096, 32768, 262144, 2097152, 16777216};
-static int M9[]={1, 9, 81, 729, 6561, 59049, 531441, 4782969, 43046721,
+static const int M8[]={1, 8, 64, 512, 4096, 32768, 262144, 2097152, 16777216};
+static const int M9[]={1, 9, 81, 729, 6561, 59049, 531441, 4782969, 43046721,
 	387420489};
 /*#define INEF2 1*/
-#ifndef INEF2 
+#ifndef INEF2
 #define TRANSITION(TT,V) V=TT&7;TT>>=3
 #define FINALTRANSITION(TT,V) V=TT&7
 #define TABLE(R,T,L,B) (table[((B)<<9)|((L)<<6)|((T)<<3)|(R)])
@@ -303,7 +314,7 @@ static int M9[]={1, 9, 81, 729, 6561, 59049, 531441, 4782969, 43046721,
 #define HEX_TABLE_IN(C,R,T,t,l,b,B,I) (HEX_TABLE(R,T,t,l,b,B)&=~(7<<((C)*3)));\
 (HEX_TABLE(R,T,t,l,b,B)|=((I)<<((C)*3)))
 #else
-#ifndef INEF2 
+#ifndef INEF2
 #define TABLE_IN(C,R,T,L,B,I) (TABLE(R,T,L,B)|=((I)<<((C)*3)))
 #else
 #define TABLE_IN(C,R,T,L,B,I) if(TABLE_OUT(C,R,T,L,B)==0) (TABLE(R,T,L,B)+=((I)*(M8[C])))
@@ -1026,8 +1037,13 @@ fillcell(ModeInfo * mi, GC gc, int col, int row)
 	if (local_neighbors == 6) {
 		int         ccol = 2 * col + !(row & 1), crow = 2 * row;
 
-		lp->shape.hexagon[0].x = lp->xb + ccol * lp->xs;
-		lp->shape.hexagon[0].y = lp->yb + crow * lp->ys;
+		if (lp->vertical) {
+			lp->shape.hexagon[0].x = lp->xb + ccol * lp->xs;
+			lp->shape.hexagon[0].y = lp->yb + crow * lp->ys;
+		} else {
+			lp->shape.hexagon[0].y = lp->xb + ccol * lp->xs;
+			lp->shape.hexagon[0].x = lp->yb + crow * lp->ys;
+		}
 		if (lp->xs == 1 && lp->ys == 1)
 			XDrawPoint(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
 				lp->shape.hexagon[0].x, lp->shape.hexagon[0].y);
@@ -1183,8 +1199,13 @@ draw_state(ModeInfo * mi, int state)
 			col = current->pt.x;
 			row = current->pt.y;
 			ccol = 2 * col + !(row & 1), crow = 2 * row;
-			lp->shape.hexagon[0].x = lp->xb + ccol * lp->xs;
-			lp->shape.hexagon[0].y = lp->yb + crow * lp->ys;
+			if (lp->vertical) {
+				lp->shape.hexagon[0].x = lp->xb + ccol * lp->xs;
+				lp->shape.hexagon[0].y = lp->yb + crow * lp->ys;
+			} else {
+				lp->shape.hexagon[0].y = lp->xb + ccol * lp->xs;
+				lp->shape.hexagon[0].x = lp->yb + crow * lp->ys;
+			}
 			if (lp->xs == 1 && lp->ys == 1)
 				XDrawPoint(display, MI_WINDOW(mi), gc,
 					lp->shape.hexagon[0].x, lp->shape.hexagon[0].y);
@@ -1383,8 +1404,6 @@ setUndefinedEvoRule()
 static Bool
 init_table(ModeInfo *mi)
 {
-	loopstruct *lp = &loops[MI_SCREEN(mi)];
-
 	if (table == NULL) {
 		int mult = 1;
 		unsigned int tt, c, n[MAXNEIGHBORS], i;
@@ -1397,30 +1416,48 @@ init_table(ModeInfo *mi)
 			sizeof (unsigned int);
 
 		if (local_neighbors == 4) {
-			if (MI_IS_FULLRANDOM(mi)) {
-				int i = NRAND(3);
+			if (MI_IS_FULLRANDOM(mi) ||
+			    !(langton || dissolve || evolve || sheath)) {
+				int i = NRAND(4);
 
 				evolve = (!i);
 				dissolve = (i == 1);
 				sheath = (!evolve && (LRAND() & 1));
 			} else {
-				dissolve = (dissolve && !evolve);
-				sheath = (sheath && !evolve);
+				evolve = (evolve && !langton);
+				dissolve = (dissolve && !evolve && !langton);
+				sheath = (sheath && !evolve && !langton);
 			}
+			langton = (!(dissolve || evolve || sheath));
 		} else {
+			langton = True;
 			evolve = False;
 			dissolve = False;
 			sheath = False;
 		}
+		if (MI_IS_VERBOSE(mi)) {
+			if (langton)
+				(void) fprintf(stdout, "Langton's Loops\n");
+			else if (evolve)
+				(void) fprintf(stdout, "Evolving Loops\n");
+			else if (dissolve && !sheath)
+				(void) fprintf(stdout, "Disolving Loops\n");
+			else if (dissolve && sheath)
+				(void) fprintf(stdout, "Disolving Sheath Loops\n");
+			else if (sheath)
+				(void) fprintf(stdout, "Sheath Loops\n");
+			else
+				(void) fprintf(stderr, "Unknown Loop\n");
+		}
 		for (j = 0; j < local_neighbors; j++)
-			mult *= (8 + (dissolve || evolve || sheath));
+			mult *= (8 + !langton);
 #if 1
 /* using space-wise inefficient method,
    uhhh I'll try to think harder on how to do it later %-) */
-/* also (!disolve && sheath) should only be multiplied by 8... here and
+/* also (!dissolve && sheath) should only be multiplied by 8... here and
    above, but lets keep it simple for now */
 /* Because of this will be using INEF_TABLE instead of EXT_TABLE */
-		if (dissolve || evolve || sheath)
+		if (!langton)
 			mult *= 9;
 #endif
 
@@ -1585,7 +1622,7 @@ init_adam(ModeInfo * mi)
 	if (!MI_COUNT(mi)) /* Probably doing testing so do not confuse */
 #endif
 	dir = NRAND(local_neighbors);
-	
+
 	if (local_neighbors == 6) {
 		int k;
 
@@ -1814,7 +1851,7 @@ do_gen(loopstruct * lp)
 	int         i, j, k;
 	unsigned char *z;
 	unsigned int n[MAXNEIGHBORS];
-	unsigned int c; 
+	unsigned int c;
 
 #if 0
 #define COLBOUND(X) ((!lp->wrap || ((X != 0) && (X != lp->bncols - 1))) ? X : ((X == 0) ? lp->bncols - 1 : 0))
@@ -1952,13 +1989,19 @@ init_loop(ModeInfo * mi)
 	if (MI_IS_FULLRANDOM(mi)) {
 
 		lp->wrap = (Bool) (LRAND() & 1);
+		lp->vertical = (Bool) (LRAND() & 1);
 	} else {
 		lp->wrap = wrap;
+		lp->vertical= vertical; 
 	}
 
   if (local_neighbors == 6) {
     int         nccols, ncrows;
 
+    if (!lp->vertical) {
+       lp->height = MI_WIDTH(mi);
+       lp->width = MI_HEIGHT(mi);
+    }
     if (lp->width < 8)
       lp->width = 8;
     if (lp->height < 8)
@@ -1989,8 +2032,13 @@ init_loop(ModeInfo * mi)
     lp->yb = (lp->height - lp->ys * ((lp->wrap) ? (ncrows / 2) * 2 : ncrows)) /
          2 + lp->ys - 2 * lp->wrap;
     for (i = 0; i < 6; i++) {
-      lp->shape.hexagon[i].x = (lp->xs - 1) * hexagonUnit[i].x;
-      lp->shape.hexagon[i].y = ((lp->ys - 1) * hexagonUnit[i].y / 2) * 4 / 3;
+      if (lp->vertical) {
+        lp->shape.hexagon[i].x = (lp->xs - 1) * hexagonUnit[i].x;
+        lp->shape.hexagon[i].y = ((lp->ys - 1) * hexagonUnit[i].y / 2) * 4 / 3;
+      } else {
+        lp->shape.hexagon[i].y = (lp->xs - 1) * hexagonUnit[i].x;
+        lp->shape.hexagon[i].x = ((lp->ys - 1) * hexagonUnit[i].y / 2) * 4 / 3;
+      }
     }
   } else {
 		if (size < -MINSIZE)
@@ -2033,7 +2081,7 @@ init_loop(ModeInfo * mi)
 	}
 	lp->mincol = lp->bncols - 1;
 	lp->minrow = lp->bnrows - 1;
-	lp->maxcol = 0; 
+	lp->maxcol = 0;
 	lp->maxrow = 0;
 #ifndef DELAYDEBUGLOOP
 	{
