@@ -870,14 +870,14 @@ fills_vertex(ModeInfo * mi, vertex_type_c vtype, fringe_node_c * vertex)
 static unsigned
 fringe_changes(ModeInfo * mi, fringe_node_c * vertex,
 	       unsigned side, vertex_type_c vtype,
-	       fringe_node_c ** right, fringe_node_c ** far,
+	       fringe_node_c ** right, fringe_node_c ** faraway,
 	       fringe_node_c ** left)
 {
 	fringe_node_c *v, *f = (fringe_node_c *) NULL;
 	unsigned    result = FC_NEW_FAR;	/* We clear this later if necessary. */
 
-	if (far)
-		*far = 0;
+	if (faraway)
+		*faraway = 0;
 	if (fills_vertex(mi, vtype, vertex)) {
 		result |= FC_CUT_THIS;
 	} else if (side == S_LEFT) {
@@ -897,8 +897,8 @@ fringe_changes(ModeInfo * mi, fringe_node_c * vertex,
 		if (fills_vertex(mi, VT_LEFT(vtype), v)) {
 			result = (result & ~FC_NEW_FAR) | FC_CUT_LEFT;
 			f = v->next;
-			if (far)
-				*far = f;
+			if (faraway)
+				*faraway = f;
 		}
 	}
 	if (!(result & FC_NEW_RIGHT)) {
@@ -908,8 +908,8 @@ fringe_changes(ModeInfo * mi, fringe_node_c * vertex,
 		if (fills_vertex(mi, VT_RIGHT(vtype), v)) {
 			result = (result & ~FC_NEW_FAR) | FC_CUT_RIGHT;
 			f = v->prev;
-			if (far)
-				*far = f;
+			if (faraway)
+				*faraway = f;
 		}
 	}
 	if (!(result & FC_NEW_FAR)
@@ -995,9 +995,9 @@ add_tile(ModeInfo * mi,
 	fringe_node_c
 		*left = (fringe_node_c *) NULL,
 		*right = (fringe_node_c *) NULL,
-		*far = (fringe_node_c *) NULL,
+		*faraway = (fringe_node_c *) NULL,
 		*node;
-	unsigned    fc = fringe_changes(mi, vertex, side, vtype, &right, &far, &left);
+	unsigned    fc = fringe_changes(mi, vertex, side, vtype, &right, &faraway, &left);
 
 	vertex_type_c
 		ltype = VT_LEFT(vtype),
@@ -1020,8 +1020,8 @@ add_tile(ModeInfo * mi,
 			if ((right = alloc_vertex(mi, vertex_dir(mi, vertex, S_LEFT) -
 					vtype_angle(vtype), vertex, tp)) == NULL)
 				return False;
-		if (far == NULL)
-			if ((far = alloc_vertex(mi, vertex_dir(mi, left, S_RIGHT) +
+		if (faraway == NULL)
+			if ((faraway = alloc_vertex(mi, vertex_dir(mi, left, S_RIGHT) +
 					vtype_angle(ltype), left, tp)) == NULL)
 				return False;
 	} else {
@@ -1029,8 +1029,8 @@ add_tile(ModeInfo * mi,
 			if ((left = alloc_vertex(mi, vertex_dir(mi, vertex, S_RIGHT) +
 					vtype_angle(vtype), vertex, tp)) == NULL)
 				return False;
-		if (far == NULL)
-			if ((far = alloc_vertex(mi, vertex_dir(mi, right, S_LEFT) -
+		if (faraway == NULL)
+			if ((faraway = alloc_vertex(mi, vertex_dir(mi, right, S_LEFT) -
 					vtype_angle(rtype), right, tp)) == NULL)
 				return False;
 	}
@@ -1042,14 +1042,14 @@ add_tile(ModeInfo * mi,
 	do {
 		if (((fc & FC_NEW_LEFT) && fived_equal(node->fived, left->fived))
 		    || ((fc & FC_NEW_RIGHT) && fived_equal(node->fived, right->fived))
-		    || ((fc & FC_NEW_FAR) && fived_equal(node->fived, far->fived))) {
+		    || ((fc & FC_NEW_FAR) && fived_equal(node->fived, faraway->fived))) {
 			/* Better luck next time. */
 			if (fc & FC_NEW_LEFT)
 				delete_vertex(mi, left, tp);
 			if (fc & FC_NEW_RIGHT)
 				delete_vertex(mi, right, tp);
 			if (fc & FC_NEW_FAR)
-				delete_vertex(mi, far, tp);
+				delete_vertex(mi, faraway, tp);
 			return False;
 		}
 		node = node->next;
@@ -1067,19 +1067,19 @@ add_tile(ModeInfo * mi,
 	}
 	if (!(fc & FC_CUT_FAR)) {
 		if (!(fc & FC_CUT_LEFT)) {
-			far->next = left;
-			left->prev = far;
+			faraway->next = left;
+			left->prev = faraway;
 		}
 		if (!(fc & FC_CUT_RIGHT)) {
-			far->prev = right;
-			right->next = far;
+			faraway->prev = right;
+			right->next = faraway;
 		}
 	}
-	draw_tile(vertex, right, far, left, vtype, mi);
+	draw_tile(vertex, right, faraway, left, vtype, mi);
 
 	/* Delete vertices that are no longer on the fringe.  Check the others. */
 	if (fc & FC_CUT_THIS) {
-		tp->fringe.nodes = far;
+		tp->fringe.nodes = faraway;
 		delete_vertex(mi, vertex, tp);
 	} else {
 		add_vtype(vertex, side, vtype);
@@ -1087,10 +1087,10 @@ add_tile(ModeInfo * mi,
 		tp->fringe.nodes = vertex;
 	}
 	if (fc & FC_CUT_FAR)
-		delete_vertex(mi, far, tp);
+		delete_vertex(mi, faraway, tp);
 	else {
-		add_vtype(far, fc & FC_CUT_RIGHT ? S_LEFT : S_RIGHT, ftype);
-		check_vertex(mi, far, tp);
+		add_vtype(faraway, fc & FC_CUT_RIGHT ? S_LEFT : S_RIGHT, ftype);
+		check_vertex(mi, faraway, tp);
 	}
 	if (fc & FC_CUT_LEFT)
 		delete_vertex(mi, left, tp);
@@ -1175,7 +1175,7 @@ legal_move(fringe_node_c * vertex, unsigned side, vertex_type_c vtype)
 static void
 add_random_tile(fringe_node_c * vertex, ModeInfo * mi)
 {
-	fringe_node_c *right, *left, *far;
+	fringe_node_c *right, *left, *faraway;
 	int         i, j, n, n_hits, n_good;
 	unsigned    side, fc, no_good, s;
 	vertex_type_c vtypes[MAX_COMPL];
@@ -1207,7 +1207,7 @@ add_random_tile(fringe_node_c * vertex, ModeInfo * mi)
 	no_good = 0;
 	n_good = n;
 	for (i = 0; i < n; i++) {
-		fc = fringe_changes(mi, vertex, side, vtypes[i], &right, &far, &left);
+		fc = fringe_changes(mi, vertex, side, vtypes[i], &right, &faraway, &left);
 		if (fc & FC_BAG) {
 			tp->done = True;
 			if (MI_IS_VERBOSE(mi)) {
@@ -1231,9 +1231,9 @@ add_random_tile(fringe_node_c * vertex, ModeInfo * mi)
 				continue;
 			}
 		}
-		if (far) {
+		if (faraway) {
 			s = ((fc & FC_CUT_LEFT) ? S_RIGHT : S_LEFT);
-			if (!legal_move(far, s, VT_FAR(vtypes[i]))) {
+			if (!legal_move(faraway, s, VT_FAR(vtypes[i]))) {
 				no_good |= (1 << i);
 				n_good--;
 			}
