@@ -80,30 +80,30 @@ static Bool trackmouse;
 #define MAXPILES (MAXALTERNATEPILES+MAXSUITS+2)
 #define OVERLAP (0.25)
 
-enum Colors {black, red};
+enum Colors {col_black, col_red};
 enum Suits {spade, diamond, club, heart};
 
 #ifdef FR
 // Ace => As, Jack => Valet,
 // Queen = Reine (Dame), King = Roi, Joker = Joker
-static const char *RankNames[] = {"", "A",
+static /*const*/ char *RankNames[] = {"", "A",
 	"2", "3", "4", "5", "6", "7", "8", "9", "10",
 	"V", "D", "R"};
 #else
 #ifdef NL
 // Ace => Aas, Jack => Boer (Farmer),
 // Queen = Vrouw (Lady), King = Heer (Gentleman), Joker = Joker
-static const char *RankNames[] = {"", "A",
+static /*const*/ char *RankNames[] = {"", "A",
 	"2", "3", "4", "5", "6", "7", "8", "9", "10",
 	"B", "V", "H"};
 #else
 #ifdef RU
 // Some may be approximations to Cyrillic
-static const char *RankNames[] = {"", "T",
+static /*const*/ char *RankNames[] = {"", "T",
 	"2", "3", "4", "5", "6", "7", "8", "9", "10",
 	"B", "Q", "K"};
 #else
-static const char *RankNames[] = {"", "A",
+static /*const*/ char *RankNames[] = {"", "A",
 	"2", "3", "4", "5", "6", "7", "8", "9", "10",
 	"J", "Q", "K"};
 #endif
@@ -225,7 +225,7 @@ public:
 inline Card::Card(Suits a_suit, int a_rank) { this->suit = a_suit; this->rank = a_rank;}
 inline Suits Card::whichSuit() { return this->suit;}
 inline int Card::whichRank() { return this->rank;}
-inline Colors Card::whichColor() { return ((whichSuit() % MAXCOLORS) ? red : black);}
+inline Colors Card::whichColor() { return ((whichSuit() % MAXCOLORS) ? col_red : col_black);}
 
 
 //cardview.h
@@ -289,7 +289,7 @@ void CardView::draw()
 	if (isFaceUp()) {
 		XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 		XFillRectangle(display, window, gc, x(), y(), width(), height());
-		if (card->whichColor() == red) {
+		if (card->whichColor() == col_red) {
 			XSetForeground(display, gc, red_pixel);
 		} else {
 			XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
