@@ -540,6 +540,9 @@ extern void release_last_mode(ModeInfo *);
 #endif
 #endif	/* UNDER_DEVELOPMENT */
 #endif	/* USE_UNSTABLE */
+#ifndef WIN32
+#define MODE_anemone
+#endif
 #define MODE_ant
 #define MODE_ant3d
 #define MODE_apollonian
@@ -569,6 +572,9 @@ extern void release_last_mode(ModeInfo *);
 #define MODE_decay
 #endif /* !WIN32 */
 #define MODE_deco
+#ifndef WIN32
+#define MODE_deluxe
+#endif /* !WIN32 */
 #define MODE_demon
 #define MODE_dilemma
 #define MODE_discrete
@@ -623,9 +629,7 @@ extern void release_last_mode(ModeInfo *);
 #define MODE_munch
 #define MODE_nose
 #define MODE_pacman
-#ifndef WIN32
 #define MODE_penrose
-#endif /* !WIN32 */
 #define MODE_petal
 #define MODE_petri
 #define MODE_polyominoes
@@ -667,9 +671,7 @@ extern void release_last_mode(ModeInfo *);
 #define MODE_wire
 #define MODE_world
 #define MODE_worm
-#ifndef WIN32
 #define MODE_xcl
-#endif /* !WIN32 */
 #define MODE_xjack
 
 #ifdef USE_BOMB
@@ -689,6 +691,7 @@ extern void release_last_mode(ModeInfo *);
 #define MODE_dclock
 #define MODE_decay
 #define MODE_deco
+#define MODE_deluxe
 #define MODE_demon
 #define MODE_dilemma
 #define MODE_dragon
@@ -747,6 +750,14 @@ extern void release_last_mode(ModeInfo *);
 
 #ifndef USE_MODULES
 
+#ifdef MODE_anemone
+extern ModeHook init_anemone;
+extern ModeHook draw_anemone;
+extern ModeHook release_anemone;
+extern ModeHook refresh_anemone;
+extern ModeSpecOpt anemone_opts;
+#endif
+
 #ifdef MODE_ant
 extern ModeHook init_ant;
 extern ModeHook draw_ant;
@@ -950,6 +961,14 @@ extern ModeHook refresh_deco;
 extern ModeSpecOpt deco_opts;
 #endif
 
+#ifdef MODE_deluxe
+extern ModeHook init_deluxe;
+extern ModeHook draw_deluxe;
+extern ModeHook release_deluxe;
+extern ModeHook refresh_deluxe;
+extern ModeSpecOpt deluxe_opts;
+#endif
+
 #ifdef MODE_demon
 extern ModeHook init_demon;
 extern ModeHook draw_demon;
