@@ -562,7 +562,7 @@ static OptionStruct opDesc[] =
 #else
 	{(char *) "-bitmap filename", (char *) "bitmap file (sometimes ras too)"},
 #endif
-	{(char *) "-erasemode erase-modename", (char *) "Erase mode to use"},
+	{(char *) "-erasemode erase-modename", (char *) "Erase mode to use (to ddisable set to no_fade)"},
 	{(char *) "-erasedelay num", (char *) "Erase delay for clear screen modes"},
 	{(char *) "-erasetime num", (char *) "Maximum time (sec) to be used by erase"},
 	{(char *) "-/+nolock", (char *) "turn on/off no password required"},
